@@ -1,10 +1,11 @@
-import os
-import string
-import re
 import binascii
-import struct
 import glob
 import logging
+import os
+import re
+import string
+import struct
+
 import vdf
 
 __all__ = (
@@ -226,6 +227,113 @@ def find_steam_runtime_path(steam_root):
     return None
 
 
+APPINFO_STRUCT_HEADER = "<4sL"
+APPINFO_STRUCT_SECTION = "<LLLLQ20sL"
+
+
+def get_appinfo_sections(path):
+    """
+    Parse an appinfo.vdf file and return all the deserialized binary VDF
+    objects inside it
+    """
+    # appinfo.vdf is not actually a (binary) VDF file, but a binary file
+    # containing multiple binary VDF sections.
+    # File structure based on comment from vdf developer:
+    # https://github.com/ValvePython/vdf/issues/13#issuecomment-321700244
+    with open(path, "rb") as f:
+        data = f.read()
+        i = 0
+
+        # Parse the header
+        header_size = struct.calcsize(APPINFO_STRUCT_HEADER)
+        magic, universe = struct.unpack(
+            APPINFO_STRUCT_HEADER, data[0:header_size]
+        )
+
+        i += header_size
+
+        if magic != b"'DV\x07":
+            raise SyntaxError("Invalid file magic number")
+
+        sections = []
+
+        section_size = struct.calcsize(APPINFO_STRUCT_SECTION)
+        while True:
+            # We don't need any of the fields besides 'entry_size',
+            # which is used to determine the length of the variable-length VDF
+            # field.
+            # Still, here they are for posterity's sake.
+            (appid, entry_size, infostate, last_updated, access_token,
+             sha_hash, change_number) = struct.unpack(
+                APPINFO_STRUCT_SECTION, data[i:i+section_size])
+            vdf_section_size = entry_size - 40
+
+            i += section_size
+            try:
+                vdf_d = vdf.binary_loads(data[i:i+vdf_section_size])
+                sections.append(vdf_d)
+            except UnicodeDecodeError:
+                # vdf is unable to decode binary VDF objects containing
+                # invalid UTF-8 strings.
+                # Since we're only interested in the SteamPlay manifests,
+                # we can skip those faulty sections.
+                #
+                # TODO: Remove this once the upstream bug at
+                # https://github.com/ValvePython/vdf/issues/20
+                # is fixed
+                pass
+
+            i += vdf_section_size
+
+            if i == len(data) - 4:
+                return sections
+
+
+def get_proton_appid(compat_tool_name, appinfo_path):
+    """
+    Get the App ID for Proton installation by the compat tool name
+    used in STEAM_DIR/config/config.vdf
+    """
+    # Parse all the individual VDF sections in appinfo.vdf to a list
+    vdf_sections = get_appinfo_sections(appinfo_path)
+
+    for section in vdf_sections:
+        if not section.get("appinfo", {}).get("extended", {}).get(
+                "compat_tools", None):
+            continue
+
+        compat_tools = section["appinfo"]["extended"]["compat_tools"]
+
+        for default_name, entry in compat_tools.items():
+            # A single compatibility tool may have multiple valid names
+            # eg. "proton_316" and "proton_316_beta"
+            aliases = [default_name]
+
+            # Each compat tool entry can also contain an 'aliases' field
+            # with a different compat tool name
+            if "aliases" in entry:
+                # All of the appinfo.vdf files encountered so far
+                # only have a single string inside the "aliases" field,
+                # but let's assume the field could be a list of strings
+                # as well
+                if isinstance(entry["aliases"], str):
+                    aliases.append(entry["aliases"])
+                elif isinstance(entry["aliases"], list):
+                    aliases += entry["aliases"]
+                else:
+                    raise TypeError(
+                        "Unexpected type {} for 'fields' in "
+                        "appinfo.vdf".format(type(aliases))
+                    )
+
+            if compat_tool_name in aliases:
+                return entry["appid"]
+
+    logger.error("Could not find the Steam Play manifest in appinfo.vdf")
+
+    return None
+
+
 def find_steam_proton_app(steam_path, steam_apps, appid=None):
     """
     Get the current Proton installation used by Steam
@@ -277,12 +385,12 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
     ]
     # Get the first name that was valid
     try:
-        name = next(name for name in potential_names if name)
+        compat_tool_name = next(name for name in potential_names if name)
     except StopIteration:
         logger.error("No Proton installation found in config.vdf")
         return None
 
-    # We've got the name,
+    # We've got the name from config.vdf,
     # now there are two possible ways to find the installation
     # 1. It's a custom Proton installation, and we simply need to find
     #    a SteamApp by its internal name
@@ -302,20 +410,11 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
     # Try option 2:
     # Find the corresponding App ID from <steam_path>/appcache/appinfo.vdf
     appinfo_path = os.path.join(steam_path, "appcache", "appinfo.vdf")
+    proton_appid = get_proton_appid(compat_tool_name, appinfo_path)
 
-    with open(appinfo_path, "rb") as f:
-        appinfo = str(binascii.hexlify(f.read()), "utf-8")
-
-    # In ASCII, the substring we're looking for looks like this
-    # ```
-    # proton_316_beta..appid.
-    # ```
-    appid_regex = "({name_ascii}0002617070696400)([a-z0-9]{{8}})".format(
-        name_ascii=str(binascii.hexlify(bytes(name, "utf-8")), "utf-8")
-    )
-    # The second group contains the App ID as a 32-bit integer in little-endian
-    proton_appid = re.search(appid_regex, appinfo).group(2)
-    proton_appid = struct.unpack("<I", binascii.unhexlify(proton_appid))[0]
+    if not proton_appid:
+        logger.error("Could not find Proton's App ID from appinfo.vdf")
+        return None
 
     # We've now got the appid. Return the corresponding SteamApp
     try:
